# SUPPORT FOR ANALYSING TIMING TESTS.

td <- read.table("test-times",head=FALSE)
colnames(td) <- c("date","time","c","t","n","conf","mod","r","platform")


# SELECT A SUBSET OF TESTS.  The first argument is the data frame with
# the test results, defaulting to the global td variable defined above.
# Other arguments are vectors of values for columns of td (except for c).  
# A data frame containing only the tests with values matching those specified 
# is returned.

sel <- function (td=get("td",1), date, time, t, n, conf, mod, r, platform)
{
  w <- rep(TRUE,nrow(td))

  if (!missing(date))     w <- w & td$date %in% date
  if (!missing(time))     w <- w & td$time %in% time
  if (!missing(t))        w <- w & td$t %in% t
  if (!missing(n))        w <- w & td$n %in% n
  if (!missing(conf))     w <- w & td$conf %in% conf
  if (!missing(mod))      w <- w & td$mod %in% mod
  if (!missing(r))        w <- w & td$r %in% r
  if (!missing(platform)) w <- w & td$platform %in% platform

  td[w,]
}


# PRINT NUMBER OF TEST RUNS BY MOD AND RUN TYPE.

run.count <- function (...)
{ 
  td <- sel(...)
  table(td[,c("mod","r")])
}


# PLOT TIME RATIOS.  

plot.ratios <- function (ts1, ts2)
{
  n <- nrow(ts1)

  ratios <- calc.ratios(ts1,ts2)

  par(mar=c(5,4,4, if (n>20) 8 else 12))
  plot (rev(ratios), 1:n, xlab="CPU time ratios (log scale)", ylab="",
        yaxt="n", pch=20, log="x") 
  abline(v=c(1,2,5,10,20,1/2,1/5,1/10,1/20))

  for (i in 1:n)
  { if (ts1$n[i] %in% 1) 
    { mtext (paste(" ",ts1$t[i]), 4, at=n+1-i, las=1, 
             cex = if (n>20) 0.5 else 1)
      abline(h=n+1.5-i,col="gray")
    }
  }
  abline(h=0.5,col="gray")
}


# CALCULATE TIME RATIOS.  Attaches names for informative printing.

calc.ratios <- function (ts1, ...)
{
  n <- nrow(ts1)
  ratios <- matrix(NA,n,0)
  rownames(ratios) <- paste(ts1$t,":",ts1$n,sep="") 

  for (ts2 in list(...))
  {
    if (nrow(ts2)!=n)
    { stop("selections not the same size")
    }

    for (i in 1:n)
    { if (ts1$t[i]!=ts2$t[i] || ts1$n[i]!=ts2$n[i])
      { stop("selections don't match:",ts1$t[i],ts1$n[i],"vs",ts2$t[i],ts2$n[i])
      }
    }

    ratios <- cbind (ratios, round(ts1$c/ts2$c,3))
  }
  
  ratios
}
