# SUPPORT FOR RUNNING TIMING TESTS.


# LOAD COMPILER.  But it's not necessarily used.

have.compiler <- as.numeric(R.version$minor)>=13
if (have.compiler) library(compiler)


# GET TEST REPETITION FACTOR.  Used to produce reasonable times on the platform
# being tested.  Default is 1, or the value of the "rep" environment variable,
# if it is set.  May be changed after this file is sourced.

test.rep <- as.numeric(system("echo $rep",intern=TRUE))

if (is.na(test.rep)) 
{ test.rep <- 1
}


# FILE TO APPEND TIMES TO.  May be changed after sourcing this file.

test.times.file <- "test-times"


# NAME AND NUMBER OF TEST BEING PERFORMED.  The name of the test should
# be changed to something suitable after sourcing this file, before each
# test is run.  The number is automatically incremented for each call of 
# the sys.time function below, and automatically reset to 1 when the 
# test name changes.

test.name <- test.name.prev <- "test"
test.number <- 0


# FUNCTION TO MAYBE COMPILE A FUNCTION.  Set to not compile (ie, to the
# identity function) by default, unless the "cmp" environment variable is
# "T", in which case it is set to cmpfun.  May be changed after this
# file is sourced.

do.not.cmpfun <- function (x) x
test.cmp <- if (have.compiler && system("echo $cmp",intern=TRUE)=="T") 
                cmpfun else do.not.cmpfun


# FIND CONFIGURATION (eg, R-2.13.0-machine) AND MOD (eg, no-mods) OF R.
# If no mod is recorded, "." is used.

this.R.config <- sub(".*/","",sub("/library","",.Library))

this.R.mod <- "."

if (system (paste ("if [ -r", sub("/library","/mod",.Library), "];",
                   "then echo T; else echo F; fi"), intern=TRUE) == "T")
{ this.R.mod <- scan (sub("/library","/mod",.Library),
                      what="character", nmax=1, quiet=TRUE)
}


# FUNCTION TO RETURN, STORE, AND PRINT USER TIME FOR EXPRESSION.  Does a
# garbage colllection beforehand for improved timing consistency.  The 
# expression is repeated test.rep times, and the time then divided by test.rep.

sys.time <- function (ex)
{ 
  tm <- system.time(
          for (i in 1:test.rep) eval.parent(substitute(ex)),
        gcFirst=TRUE)[1] 
  tm <- tm / test.rep

  if (test.name!=test.name.prev)
  { test.number <<- 0
    test.name.prev <<- test.name
    cat("\n----",test.name,"\n\n")
  }
  test.number <<- test.number + 1

  res <- paste (format(round(tm,3),nsmall=3,width=8), "  ", 
    test.name, test.number, this.R.config, this.R.mod,
    paste (if (identical(test.cmp,do.not.cmpfun)) "I" else "C", 
           test.rep, sep=""))

  a <- file("test-times","a")
  writeLines (paste (substr(Sys.time(),1,19), res, R.version$platform), a)
  close(a)

  cat ("Time:", res, "\n")

  tm
}
